package com.usky.common.tenant.core.db;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.handler.TenantLineHandler;
import com.usky.common.tenant.config.TenantProperties;
import com.usky.common.tenant.core.context.TenantContextHolder;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.LongValue;

import java.util.HashSet;
import java.util.Set;



/**
 * 基于 MyBatis Plus 多租户的功能，实现 DB 层面的多租户的功能
 * TenantDatabaseInterceptor类实现了TenantLineHandler接口，并实现了
 * getTenantId()方法，该方法主要用于设置租户Id的值，在框架去处理SQL语句前去改写SQL语句，
 * 为SQL语句添加上租户判断条件。租户Id可以从缓存、cookie、token等中获取（根据实际的业务场景来）
 * getTenantIdColumn()方法，该方法用于设置租户Id的字段名称
 * ignoreTable(String tableName)方法，该方法用于标记忽略添加租户ID的表
 * @author yq
 */
public class TenantDatabaseHandler implements TenantLineHandler {

    /**
     * 忽略添加租户ID的表
     */
    private final Set<String> ignoreTables = new HashSet<>();

    public TenantDatabaseHandler(TenantProperties properties) {
        // 不同 DB 下，大小写的习惯不同，所以需要都添加进去
        if (CollectionUtils.isNotEmpty(properties.getIgnoreTables())){
            properties.getIgnoreTables().forEach(table -> {
                ignoreTables.add(table.toLowerCase());
                ignoreTables.add(table.toUpperCase());
            });
        }
        // 在 OracleKeyGenerator 中，生成主键时，会查询这个表，查询这个表后，会自动拼接 TENANT_ID 导致报错
        ignoreTables.add("DUAL");
    }

    @Override
    public Expression getTenantId() {
        return new LongValue(TenantContextHolder.getRequiredTenantId());
    }

    @Override
    public boolean ignoreTable(String tableName) {
        // 情况一，全局忽略多租户
        // 情况二，忽略多租户的表
        return TenantContextHolder.isIgnore()
            || ignoreTables.stream().anyMatch(s -> s.equals(tableName));
    }

}
