package com.usky.common.tenant.core.web;


import com.usky.common.tenant.core.context.TenantContextHolder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * 多租户 Context Web 过滤器
 * 将请求 Header 中的 tenant_id 解析出来，添加到 {@link TenantContextHolder} 中，这样后续的 DB 等操作，可以获得到租户编号。
 * 直接从Header获取tenant_id风险和优势：
 * 1.放到header的tentant_id有被篡改的风险，考虑到降低组件依赖耦合，该行为应该有其它组件来预防实现，它的主要职责进行横切拼接SQL
 * @author yq
 */
@Slf4j
public class TenantContextWebFilter extends OncePerRequestFilter {

    /**
     * 租户ID KEY
     */
    private final static String HEADER_TENANT_ID = "tenant_id";

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain chain)
            throws ServletException, IOException {
        // 设置
        Integer tenantId = getTenantId(request);
        log.debug("tenantId value:{}",tenantId);
        if (tenantId != null) {
            TenantContextHolder.setTenantId(tenantId);
        }
        if (tenantId == null) {
            TenantContextHolder.setIgnore(true);
        }
        try {
            chain.doFilter(request, response);
        } finally {
            // 清理
            TenantContextHolder.clear();
        }
    }


    /**
     * 后续考虑设置web模块从web模块里面来获取
     */
    public static Integer getTenantId(HttpServletRequest request) {
        String tenantId = request.getHeader(HEADER_TENANT_ID);
        return StringUtils.isNotEmpty(tenantId) ? Integer.valueOf(tenantId) : null;
    }

}
